@@ -1,4 +1,5 @@
 use std::collections::HashMap;
+use std::hash::{Hash, Hasher};
 
 use num_enum::{IntoPrimitive, TryFromPrimitive};
 use uuid::Uuid;
@@ -191,7 +192,7 @@ impl BrickOwner {
 }
 
 /// A color, in RGBA.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Color {
     pub r: u8,
     pub g: u8,
@@ -305,6 +306,7 @@ pub struct Brick {
     pub material_intensity: u32,
 
     /// The color of the brick. When referring to an index from the colors array in `Header2`, use `BrickColor::Index`. Otherwise, use `BrickColor::Unique(Color)`.
+    #[cfg_attr(feature = "serialize", serde(serialize_with = "brick_color_serialize"))]
     pub color: BrickColor,
 
     /// The owner index of the brick. When 0, this brick's owner is PUBLIC. Otherwise, it refers to `Header2`'s `brick_owners`, 1-indexed.
@@ -314,6 +316,20 @@ pub struct Brick {
     pub components: HashMap<String, HashMap<String, UnrealType>>,
 }
 
+#[cfg(feature = "serialize")]
+fn brick_color_serialize<S: Serializer>(color: &BrickColor, s: S) -> Result<S::Ok, S::Error> {
+    match color {
+        BrickColor::Index(index) => s.serialize_u32(*index),
+        BrickColor::Unique(color) => {
+            let mut tup = s.serialize_tuple(3)?;
+            tup.serialize_element(&color.r)?;
+            tup.serialize_element(&color.g)?;
+            tup.serialize_element(&color.b)?;
+            tup.end()
+        }
+    }
+}
+
 impl Default for Brick {
     fn default() -> Self {
         Brick {
@@ -334,6 +350,23 @@ impl Default for Brick {
     }
 }
 
+impl Hash for Brick {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.asset_name_index.hash(state);
+        self.size.hash(state);
+        self.position.hash(state);
+        self.direction.hash(state);
+        self.rotation.hash(state);
+        self.collision.hash(state);
+        self.visibility.hash(state);
+        self.material_index.hash(state);
+        self.physical_index.hash(state);
+        self.material_intensity.hash(state);
+        self.color.hash(state);
+        self.owner_index.hash(state);
+    }
+}
+
 /// Represents a brick's direction.
 #[repr(u8)]
 #[derive(Debug, Clone, IntoPrimitive, TryFromPrimitive, PartialEq, Eq, PartialOrd, Ord, Hash)]
@@ -362,7 +395,7 @@ pub enum Rotation {
 ///
 /// Procedural bricks should use `Size::Procedural`.
 /// Static mesh bricks should use `Size::Empty`.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub enum Size {
     /// A singularity (used for non-procedural bricks).
     Empty,
@@ -428,7 +461,7 @@ impl<'de> Deserialize<'de> for Size {
 ///
 /// Bricks that refer to a color in their save should use `BrickColor::Index`.
 /// Bricks defining their own `Color` should use `BrickColor::Unique`.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serialize", derive(Serialize, Deserialize), serde(untagged))]
 pub enum BrickColor {
     /// A color that links to an index in the save palette.
@@ -439,7 +472,7 @@ pub enum BrickColor {
 }
 
 /// Represents a brick's collision flags.
-#[derive(Debug, Clone, PartialEq, Eq)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serialize", derive(Serialize, Deserialize), serde(default))]
 pub struct Collision {
     pub player: bool,
